/***************************************************************************
 *   Copyright (C) 2005 by Florian Zubeil   *
 *   daswrack@oleco.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "BrowseWindow.h"

BrowseWindow::BrowseWindow( QSettings& settings_, QWidget* parent , const char* name )
: BrowseWindowBase( parent, name  ),
  settings(settings_),
  Io(0)
{
    QValueList<int> bsSizes;
    bsSizes.append(300);
    bsSizes.append(500);
    browseSplitter->setSizes(bsSizes);
    listView1->clear();
    listView2->clear();
    //XmlIo::load( listView1, "bla.xml");
    if (!QObject::connect(listView1, SIGNAL( selectionChanged ( QListViewItem* ) ), this, SLOT(dirClicked( QListViewItem* )) ))
        ACE_ERROR((LM_ERROR,ACE_TEXT("%N:%l: Error connecting Signal 'clicked' and slot 'dirClicked'!")));
    
    if (!QObject::connect(listView2, SIGNAL( selectionChanged ( QListViewItem* ) ), this, SLOT(fileClicked( QListViewItem* )) ))
        ACE_ERROR((LM_ERROR,ACE_TEXT("%N:%l: Error connecting Signal 'clicked' and slot 'fileClicked'!")));

    this->setIcon( QPixmap( QPixmap( QPixmap::fromMimeSource("app_icon.png") ) ) );
    
    //Create preview DockWindow and create the TextEdit as child
    previewDockWindow   = new   QDockWindow     ( QDockWindow::InDock, this->bottomDock() );
    previewTextEdit     = new   QTextEdit       ( previewDockWindow );
    previewTextEdit     ->      setReadOnly     ( true );
    previewTextEdit     ->      setText         ( tr("No file selected.") );
    previewDockWindow   ->      setResizeEnabled( true );
    previewTextEdit     ->      resize          ( 500, 100 );
    previewDockWindow   ->      setWidget       ( previewTextEdit );
    previewDockWindow   ->      setHorizontallyStretchable( true );
    previewDockWindow   ->      setVerticallyStretchable( false );
    previewDockWindow   ->      setFixedExtentWidth( 250 );
    previewDockWindow   ->      setCloseMode    ( QDockWindow::Always );
    previewDockWindow   ->      setCaption      ( tr("File Preview") );


    //Set Icons for Actions
    fileNewAction->setIconSet   ( QIconSet( QPixmap::fromMimeSource("filenew.png"    ) ) );
    fileOpenAction->setIconSet  ( QIconSet( QPixmap::fromMimeSource("folder.png"    ) ) );
    fileSaveAction->setIconSet  ( QIconSet( QPixmap::fromMimeSource("filesave.png"  ) ) );
    fileSaveAsAction->setIconSet( QIconSet( QPixmap::fromMimeSource("filesaveas.png") ) );
    fileScanAction->setIconSet  ( QIconSet( QPixmap::fromMimeSource("viewmag.png"   ), QPixmap::fromMimeSource("viewmag24.png"   ) ) );
    fileExitAction->setIconSet  ( QIconSet( QPixmap::fromMimeSource("exit.png"      ) ) );

    toolNewAction = new QAction(this);
    toolOpenAction = new QAction(this);
    toolSaveAction = new QAction(this);
    toolSaveAsAction = new QAction(this);
    toolScanAction = new QAction(this);
    toolNewAction->setIconSet( QIconSet( QPixmap::fromMimeSource("filenew24.png") ) );
    toolOpenAction->setIconSet( QIconSet( QPixmap::fromMimeSource("folder24.png") ) );
    toolSaveAction->setIconSet( QIconSet( QPixmap::fromMimeSource("filesave24.png") ) );
    toolSaveAsAction->setIconSet( QIconSet( QPixmap::fromMimeSource("filesaveas24.png") ) );
    toolScanAction->setIconSet( QIconSet( QPixmap::fromMimeSource("viewmag24.png") ) );
    toolNewAction->addTo( toolBar );
    toolOpenAction->addTo( toolBar );
    toolSaveAction->addTo( toolBar );
    toolSaveAsAction->addTo( toolBar );
    toolScanAction->addTo( toolBar );

    QObject::connect( toolNewAction, SIGNAL( activated() ), this, SLOT( fileNew() ) );
    QObject::connect( toolOpenAction, SIGNAL( activated() ), this, SLOT( fileOpen() ) );
    QObject::connect( toolSaveAction, SIGNAL( activated() ), this, SLOT( fileSave() ) );
    QObject::connect( toolSaveAsAction, SIGNAL( activated() ), this, SLOT( fileSaveAs() ) );
    QObject::connect( toolScanAction, SIGNAL( activated() ), this, SLOT( scan() ) );
    this->applySettings();
}

BrowseWindow::~BrowseWindow()
{
    this->saveSettings();
    delete toolSaveAction;
    delete toolSaveAsAction;
    delete toolScanAction;
    delete Io;
}

void BrowseWindow::dirClicked( QListViewItem *item )
{
    if ( !item || item->rtti() != RTTI_DIRVIEWITEM )
        return;

    DirViewItem*    viewItem    = (DirViewItem*)    item;
    DirItem*        dirItem     = (DirItem*)        viewItem->getDir();
    listView2->clear();
    dirItem->addFileItems( listView2 );
}

void BrowseWindow::fileClicked( QListViewItem *item )
{
    if ( !item || item->rtti() != RTTI_FILEVIEWITEM )
        return;

    FileViewItem*    viewItem   = (FileViewItem*)    item;
    previewTextEdit->setText( viewItem->getInfoText() );
}

void BrowseWindow::scan()
{
    StatusDialog sDialog( settings, *listView1, *listView2, this, "bla", TRUE  );
    sDialog.exec();
    if ( sDialog.wasSuccessful() )
        fileSaveAction->setEnabled( true );
}

void BrowseWindow::fileNew()
{
    listView1->clear();
    delete Io;
}

void BrowseWindow::fileOpen()
{
    QString lFilename = QFileDialog::getOpenFileName( QString::null,
                                                      tr("XML-Files") + " (*.xml)",
                                                      this,
                                                      tr("load file dialog"),
                                                      tr("Choose a filename to load"));
    if( !lFilename.isNull() ) {
        QString pass;
        InputDialog* id = new InputDialog( &pass, tr("Enter password"), tr("Please enter password: "), "decrypted_big.png" );
        id->exec();
        delete id;
        
        if( Io )
            delete Io;
        Io = new XmlIo( listView1, lFilename, pass );
	listView1->clear();
        Io->load();
    }

}

void BrowseWindow::fileSave()
{
    if( Io ) {
        Io->save();
        fileSaveAction->setEnabled( false );
    } else
	this->fileSaveAs();

}

void BrowseWindow::fileSaveAs()
{
    QString sFilename = QFileDialog::getSaveFileName( tr("~"),
                                                      tr("XML-Files") + " (*.xml)",
                                                      this,
                                                      tr("save file dialog"),
                                                      tr("Choose a filename to save under"));
    if ( sFilename != QString::null ) {
        QString pass;
        InputDialog* id = new InputDialog( &pass, tr("Enter password"), tr("Please enter a password for this file: "), "encrypted_big.png" );
        id->exec();
        delete id;

        if( Io )
            Io->save( sFilename, pass );
	else {
	    Io = new XmlIo( listView1, sFilename, pass ); 
	    Io->save();
	}
	
    }
}

void BrowseWindow::fileSettings()
{
    ConfigDialog d(settings);
    d.exec();
}

void BrowseWindow::fileExit()
{
    exit(0);
}

void BrowseWindow::startSearch()
{
    if ( searchEdit->text() == "" ) {
        ACE_ERROR((LM_ERROR,ACE_TEXT("%N:%L: Error: Search string is empty!\n")));
        return;
    }
    SearchResults* searchRes = new SearchResults;
    resultTable->setNumRows(0);

    QListViewItem* viewItem = listView1->firstChild();
    while ( viewItem ) {
        if( viewItem->rtti() == RTTI_DIRVIEWITEM )
        {
            DirViewItem *item = (DirViewItem*)viewItem;
            DirItem* dirItem = (DirItem*)item->getDir();
            QRegExp expr( searchEdit->text(), caseCheckBox->isChecked() );
            if ( !dirItem->searchFile( &expr, searchRes ) ) {
                cout << "Error searching file!" << endl;
            }
        }
        viewItem = viewItem->nextSibling();
    }
    resultTable->setResults( searchRes );
    
    for( int i=0; i<resultTable->numCols(); i++ ) 
	resultTable->adjustColumn(i);

}

void BrowseWindow::helpAbout() {
    AboutDialogBase about;
    about.exec();
}

void BrowseWindow::applySettings()
{
    settings.beginGroup("/dvdindex");
    settings.beginGroup("/MainWindow");
    settings.beginGroup("/Geometry");
    int width_ =  settings.readNumEntry( "Width" );
    int height_ = settings.readNumEntry( "Height" );
    int x_ = settings.readNumEntry( "PosX" );
    int y_ = settings.readNumEntry( "PosY" );
    if( ( width_ > 0 ) && (height_ > 0 ) )
        setGeometry( QRect( x_, y_, width_, height_ ) );
    
    QValueList<int> sizes;
    QStringList sizeEntries = settings.entryList( "/Splitter" );
    settings.beginGroup("/Splitter");
    
    for( QStringList::iterator it = sizeEntries.begin();
         it != sizeEntries.end();
         ++it )
    {
        sizes.append(settings.readNumEntry( *it ));
    }
    settings.endGroup();
    browseSplitter->setSizes( sizes );
     
    settings.endGroup();
    settings.endGroup();
    settings.endGroup();
    
    
}

void BrowseWindow::saveSettings()
{
    settings.beginGroup("/dvdindex");
    settings.beginGroup("/MainWindow");
    settings.beginGroup("/Geometry");
    settings.writeEntry("Width", width());
    settings.writeEntry("Height", height());
    settings.writeEntry("PosX", x() );
    settings.writeEntry("PosY", y() );
    settings.beginGroup("/Splitter");
    QValueList<int> sizes = browseSplitter->sizes();
    QValueList<int>::Iterator it;
    int n = 0;
    for( it = sizes.begin(); it != sizes.end(); ++it ) {
        settings.writeEntry( QString().setNum( n ), *it );
        n++;
    }
    settings.endGroup();
    settings.endGroup();
    settings.endGroup();
    settings.endGroup();
}
