/***************************************************************************
 *   Copyright (C) 2005 by Florian Zubeil   *
 *   daswrack@oleco.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ConfigDialog.h"

#include <iostream>
using namespace std;

ConfigDialog::ConfigDialog( QSettings& settings_, QWidget* parent )
    : ConfigDialogBase( parent ), settings(settings_)
{
    //Hide the tables headers
    pluginTable->horizontalHeader()->hide();
    pluginTable->verticalHeader()->hide();
    pluginTable->setTopMargin(0);
    pluginTable->setLeftMargin(0);
    pluginTable->setNumCols(4);
    //pluginTable->setReadOnly( true );
    this->loadSettings();
}


ConfigDialog::~ConfigDialog()
{
}

void ConfigDialog::accept()
{
    this->saveSettings();
    done(0);
}

void ConfigDialog::reject()
{
    done(1);
}

void ConfigDialog::browsePlugins()
{
    QString pDir = QFileDialog::getExistingDirectory();
    if( pDir != QString::null ) {
        pluginLocationEdit->setText( pDir );
        this->rescanPlugins();
    }
}

void ConfigDialog::rescanPlugins()
{
    QDir dir(pluginLocationEdit->text());
    if( !dir.exists() )
        return;
    
    QStringList files = dir.entryList( QDir::Files );
    QStringList plugins;
    int i = 0;
    pluginTable->setNumRows( 0 );
    for( QStringList::iterator it = files.begin(); it != files.end(); ++it ) {
        QLibrary lib(dir.absPath() + QDir::separator() + *it );
        getNameProc pluginName = (getNameProc) lib.resolve("getName");
        if( !pluginName )
            continue;
        // Dont list a plugin twice
        if( plugins.contains( pluginName() ) > 0 )
            continue;
        plugins.append( pluginName() );
        
        //Incrementing numRows is slow, but should be OK here
        pluginTable->setNumRows( pluginTable->numRows()+1 );
        
        getDescProc pluginDesc = (getDescProc) lib.resolve("getDesc");
        if( !pluginDesc )
            continue;
        else
            pluginTable->setText( i, 1, pluginDesc() );
        
        getAuthorProc pluginAuthor = (getAuthorProc) lib.resolve("getAuthor");
        if( !pluginAuthor)
            continue;
        else
            pluginTable->setText( i, 3, pluginAuthor() );
        
        getSupportedTypesProc pluginTypes = (getSupportedTypesProc) lib.resolve("getSupportedTypes");
        if( !pluginTypes)
            continue;
        else
            pluginTable->setText( i, 2, pluginTypes() );
        
        pluginTable->setItem( i, 0, new PluginCheckItem( pluginTable, *it ) );
        
        for( int n=0; n<4; n++)
            pluginTable->adjustColumn(n);
        
        i++;
    }
    
}

void ConfigDialog::loadSettings()
{
    settings.beginGroup("/dvdindex");
    settings.beginGroup("/Plugins");
    pluginLocationEdit->setText( settings.readEntry("Path") );
    this->rescanPlugins();
    //Set the enabled plugins checked
    QStringList plugins = settings.entryList( "/Enabled" );
    for( QStringList::iterator it = plugins.begin(); it != plugins.end(); ++it )
        for( int i=0; i < pluginTable->numRows(); i++ ) 
            if( pluginTable->item( i, 0 ) && (pluginTable->item( i, 0 )->rtti() == RTTI_PLUGINCHECKITEM ) )
                if( ((PluginCheckItem*)( pluginTable->item( i, 0 ) ))->getFilename() == *it )
                    ((PluginCheckItem*)( pluginTable->item( i, 0 ) ))->setChecked( true );
    settings.endGroup();
    settings.endGroup();
    
            

}

void ConfigDialog::saveSettings()
{

    
    settings.beginGroup("/dvdindex");
    settings.beginGroup("/Plugins");
    
    if( pluginLocationEdit->text().length() > 0 )
        settings.writeEntry( "Path", pluginLocationEdit->text() );
    
    QStringList plugins = settings.entryList( "/Enabled" );
    settings.beginGroup("/Enabled");
    for( QStringList::iterator it = plugins.begin(); it != plugins.end(); ++it )
        settings.removeEntry( *it );
    for( int i = 0; i < pluginTable->numRows(); i++) {
        if( pluginTable->item( i, 0 ) && (pluginTable->item( i, 0 )->rtti() == RTTI_PLUGINCHECKITEM ) ) {
            if( ((PluginCheckItem*)( pluginTable->item( i, 0 ) ))->isChecked() ) {
                settings.writeEntry( 
                        ((PluginCheckItem*)( pluginTable->item( i, 0 ) ))->getFilename(), true );
            }
        }
    }
    settings.endGroup();
    settings.endGroup();
    settings.endGroup();
}
