/***************************************************************************
 *   Copyright (C) 2005 by Florian Zubeil   *
 *   daswrack@oleco.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "DirItem.h"
#include "settings.h"

DirItem::DirItem( QDomNode *n, QString name_, QString discName_, bool isRoot_ ) : 
    isRoot(isRoot_), 
    name(name_), 
    discName(discName_)
{
    QDomNode r = n->firstChild();
    for( int i=0; !r.isNull(); i++)
    {
        if( r.nodeName() == "SubDir" )
        {
	    DirItem* dir = new DirItem(&r, r.toElement().attribute("Name"), discName );
            addDir(dir);
        } else if ( r.nodeName() == "File") {
            FileItem* file = new FileItem( r.toElement().attribute("Name"), r.toElement().attribute("Size").toLong(), r.toElement().attribute("Info"));
            addFile( file );

        }
        r = r.nextSibling();
    }
}

DirItem::~DirItem()
{
    if(files.count() > 0 ) {
        QIntDictIterator<FileItem> it( files );
        for ( ; it.current(); ++it )
            delete it.current();
    }
}

int DirItem::addListViewItem( QListViewItem* item_ )
{
    DirViewItem* vItem = new DirViewItem( item_, this, name );
    vItem->setPixmap( 0, QPixmap( QPixmap::fromMimeSource("folder.png") ) );

    if(dirs.count() > 0 ) {
        QIntDictIterator<DirItem> it( dirs );
        for ( ; it.current(); ++it )
            it.current()->addListViewItem(vItem);
    }

    return 0;
}

int DirItem::addListViewItem( QListView* view_ )
{
    if( !isRoot ) 
        ACE_ERROR_RETURN((LM_ERROR,ACE_TEXT("Error: A QListView has been passed although this dir is not root!")),-1);

    DirViewItem* vItem = new DirViewItem( view_, this, name );
    vItem->setPixmap( 0, QPixmap( QPixmap::fromMimeSource("disc.png") ) );
    


    if(dirs.count() > 0 ) {
        
        QIntDictIterator<DirItem> it( dirs );
        for ( ; it.current(); ++it )
            it.current()->addListViewItem(vItem);
    }


    return 0;
}

int DirItem::addFileItems( QListView *view_ )
{
    if(files.count() > 0 ) {

        QIntDictIterator<FileItem> it( files );
        for ( ; it.current(); ++it )
            new FileViewItem( view_ , it.current(), it.current()->getName() );
    } else
        cout << "no files" << endl;
    return 1;
}

int DirItem::addToDom( QDomElement* element, QDomDocument* doc )
{

    if( !element )
        ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("%N:%l: Error: 'element' is empty!\n")),-1);

    if( !doc )
        ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("%N:%l: Error: 'doc' is empty!\n")),-1);
    //ACE_DEBUG((LM_DEBUG,ACE_TEXT("%N:&L: Adding to Dom: '%C'\n"),name.latin1()));

    if(dirs.count() > 0 ) {
        
        QIntDictIterator<DirItem> it( dirs );
        for ( ; it.current(); ++it ) {
            QDomElement subdir  = doc->createElement("SubDir");
            QDomAttr    aName   = doc->createAttribute("Name");

            aName.setValue( it.current()->getName() );
            it.current()->addToDom( &subdir, doc );
            subdir.setAttributeNode( aName );
            element->appendChild( subdir );
        }
    }

    if(files.count() > 0 ) {

        QIntDictIterator<FileItem> it( files );
        for ( ; it.current(); ++it ) {
            QDomElement file    = doc->createElement("File");
            QDomAttr    aName   = doc->createAttribute("Name");
            QDomAttr    aSize   = doc->createAttribute("Size");

            aName.setValue        ( it.current()->getName() );
            aSize.setValue        ( QString().setNum(it.current()->getSize()) );
            file.setAttributeNode ( aName );
            file.setAttributeNode ( aSize );

            if( it.current()->getInfo().length() > 0 ) {
                QDomAttr aInfo  = doc->createAttribute("Info");
                aInfo.setValue( it.current()->getInfo() );
                file.setAttributeNode( aInfo );
            }
            element->appendChild  ( file );
        }
    }
    return 1;
}

int DirItem::searchFile( QRegExp* expr, SearchResults* searchRes )
{
    if( !searchRes )
        ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("%N:%l: Error: searchRes is not valid!\n")),-1);

    ACE_DEBUG((LM_DEBUG,ACE_TEXT("%N:%l: Searching in dir '%C'\n"),name.latin1()));

    if(files.count() > 0 ) {
        QIntDictIterator<FileItem> it( files );
        for ( ; it.current(); ++it ) {
            if( it.current()->getName().find( (*expr), 0 ) != -1 ) {
                //resultTable->setNumRows( resultTable->numRows()+1 );
                //resultTable->setText( resultTable->numRows()-1, 0, it.current()->getName() );
                //resultTable->setText( resultTable->numRows()-1, 2, QString().setNum(it.current()->getSize()) );
                searchRes->addResult( new SearchResultItem( it.current()->getName(), discName, it.current()->getSize(), true ) );
            }
        }
    }

    if(dirs.count() > 0 ) {
        
        QIntDictIterator<DirItem> it( dirs );
        for ( ; it.current(); ++it ) 
            it.current()->searchFile( expr, searchRes );
    }

    return 1;
}



