/***************************************************************************
 *   Copyright (C) 2005 by Florian Zubeil   *
 *   daswrack@oleco.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __DIRITEM_H__
#define __DIRITEM_H__
#include <iostream>

#include <qintdict.h>
#include <qlistview.h>
#include <qdom.h>
#include <qtable.h>
#include <qregexp.h>
#include <qpixmap.h>

#include "FileViewItem.h"
#include "FileItem.h"
#include "DirViewItem.h"
#include "SearchResults.h"

//#include "settings.h"


using namespace std;

class DirItem : public QObject
{
    Q_OBJECT
public:
    inline DirItem( QString name_, QString discName_ ) : 
    		parent(0), 
    		isRoot(true), 
    		name(name_), 
    		discName( discName_ ) {};

    inline DirItem( QString name_, QString discName_, DirItem& parent_ ) : 
    		parent(&parent_), 
    		isRoot(false), 
    		name(name_), 
    		discName(discName_) { files.setAutoDelete(true); };

    DirItem( QDomNode* n, QString name_, QString discName_, bool isRoot_ = false );
    ~DirItem();

    inline int addFile( FileItem* file_ ) {
	files.insert( files.count(), file_ );
	return 1;
    }

    inline int addDir( DirItem* dir_ ) {
        dirs.insert(dirs.count(), dir_);
        return 1;
    }


    int addListViewItem( QListViewItem* item_ );
    int addListViewItem( QListView* view_ );
    int addFileItems( QListView* view_ );
    int addToDom( QDomElement* element, QDomDocument* doc );
    inline QString getName() { return name; }
    int searchFile( QRegExp* expr, SearchResults* searchRes );

private:
    DirItem* parent;
    QIntDict<FileItem> files;
    QIntDict<DirItem> dirs;
    bool isRoot;
    QString name;
    QString discName;
 };

#endif //__DIRITEM_H__
