/***************************************************************************
 *   Copyright (C) 2005 by Florian Zubeil   *
 *   daswrack@oleco.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "FileReadThread.h"
#include <iostream>
#include <qapplication.h>

using namespace std;

FileReadThread::FileReadThread( QSettings& settings_,
                                QString baseDir,
                                QString discName_,
                                QListView* dirView_,
                                QListView* fileView_,
                                QObject* mainWin_,
                                StatusDialogBase* statusDialog_,
                                bool *abort_ ) : 
                settings( settings_ ),
                dirItem(0),
                dir(baseDir),
                dirView(dirView_),
                fileView(fileView_),
                mainWin(mainWin_),
                sDialog(statusDialog_),
                abort(abort_),
                discName( discName_ )
                
{
    dirItem = new DirItem(discName, discName);
}

/// Main function of this Thread
void FileReadThread::run()
{
    //sDialog->show();
    sDialog->setEnabled( false );
    sDialog->setTotalFiles( countFiles(dir) );
    this->createPluginMap();
    if(!readDir(dir,dir.absPath(),dirItem,true) ) {
        //ACE_ERROR_RETURN((LM_ERROR,ACE_TEXT("%N:%l: Error reading dir!")),-1);
        sDialog->setSuccessful( false );
        return;	
    }

    dirItem->addListViewItem(dirView);
    if (!QObject::connect(
         dirView, 
         SIGNAL( clicked ( QListViewItem* ) ),
         mainWin,
         SLOT(dirClicked( QListViewItem* )) )
       )
        ACE_ERROR((LM_ERROR,ACE_TEXT("%N:%l: Error connecting Signal 'clicked' and slot 'dirClicked'!")));
    
    if(dirItem)
        dirItem->addFileItems(fileView);
    
    sDialog->setEnabled( true );
    sDialog->setProgress( 0 );
}

bool FileReadThread::readDir(QDir &dir, QString root, DirItem *parent, bool first )
{
    if( *abort || !dir.exists() )
        return false;

    DirItem *dirItem;
    if(first)
        dirItem = parent;
    else {
        dirItem = new DirItem(dir.dirName(), discName, *parent);
        parent->addDir(dirItem);
    }
    QStringList subdirs = dir.entryList( QDir::Dirs );
    QStringList filenames = dir.entryList(QDir::Files);

    for( QStringList::iterator it = filenames.begin(); it != filenames.end(); ++it )
    {
        postEvent(sDialog, new QEvent(QEvent::User));
        FileItem* file = new FileItem( pluginMap, *it, dir.absPath() + "/", QFile( dir.absFilePath( *it ) ).size() );
        dirItem->addFile( file );
    }
    //Don't go a level up...
    subdirs.remove(".");
    subdirs.remove("..");
    subdirs.remove("lost+found");

    if( !subdirs.empty() ) {
        for( QStringList::iterator it = subdirs.begin(); it != subdirs.end(); ++it )
        {
            QDir subdir(dir.absPath() + "/" + (*it));
            if(!readDir(subdir,root,dirItem))
                return false;
        }
    }

    return true;
}

void FileReadThread::createPluginMap()
{
    settings.beginGroup("/dvdindex");
    settings.beginGroup("/Plugins");
    QString path = settings.readEntry("Path");
    QStringList plugins = settings.entryList("/Enabled");
    cout << plugins.join(";") << endl;
    for( QStringList::iterator it = plugins.begin(); it != plugins.end(); ++it )
    {
        QLibrary lib( (path + (*it) ) );
        getSupportedTypesProc pluginTypes = (getSupportedTypesProc) lib.resolve("getSupportedTypes");
        if( pluginTypes ) {
            QStringList types = QStringList::split(';',pluginTypes());
            for (QStringList::iterator it2 = types.begin(); it2 != types.end(); ++it2) {
                if( !pluginMap.contains( *it2 ) )
                    pluginMap[ (*it2).lower() ] = path + (*it);
            }
        } 
    }
    settings.endGroup();
    settings.endGroup();
}
