/***************************************************************************
 *   Copyright (C) 2005 by Florian Zubeil   *
 *   daswrack@oleco.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __FILEREADTHREAD_H__
#define __FILEREADTHREAD_H__

#include <qstring.h>
#include <qdir.h>
#include <qmainwindow.h>
#include <qlistview.h>
#include <qthread.h>
#include <qmap.h>
#include <qsettings.h>
#include <qstringlist.h>
#include <qlibrary.h>

#include "plugin.h"
#include "FileItem.h"
#include "DirItem.h"
#include "StatusDialog.h"


class FileReadThread : public QThread
{
public:
    FileReadThread( QSettings& settings_,
                    QString baseDir, 
                    QString discName_, 
                    QListView* dirView_, 
                    QListView* fileView_, 
                    QObject* mainWin_, 
                    StatusDialogBase* statusDialog_, 
                    bool *abort_ );
    
    virtual void run();

private:
    bool readDir( QDir &dir, QString root, DirItem* parent = 0, bool first = false );

    ///This function determines the number of files of QDir dir (recursive)
    inline int countFiles(QDir &dir)
    {
        int files = 0;
        QStringList subdirs = dir.entryList( QDir::Dirs );
        dir.setFilter( QDir::Files );
        files += dir.count();

        //Don't go a level up...
        subdirs.remove(".");
        subdirs.remove("..");
        subdirs.remove("lost+found");

        if( !subdirs.empty() ) {
            for( QStringList::iterator it = subdirs.begin(); it != subdirs.end(); ++it )
            {
                QDir subdir(dir.absPath() + "/" + (*it));
                files += countFiles(subdir);
            }
        }
        return files;
    }
    
    void createPluginMap();
    
    
    QSettings&          settings;
    DirItem*            dirItem;
    QDir                dir;
    QListView*          dirView;
    QListView*          fileView;
    QObject*            mainWin;
    StatusDialogBase*   sDialog;
    bool*               abort;
    QString             discName;
    QMap<QString,QString> pluginMap;

};

#endif
