/***************************************************************************
 *   Copyright (C) 2005 by Florian Zubeil   *
 *   daswrack@oleco.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __SEARCHRESULTS_H__
#define __SEARCHRESULTS_H__

#include "DirViewItem.h"
#include "SizeTableItem.h"
//#include "ace/Containers.h"
#include <qptrlist.h>
#include <qstring.h>
#include <qtable.h>

#include <stdlib.h>
#include <iostream>

using namespace std;

class SearchResultItem
{
    public:
    SearchResultItem( QString name_, QString disc_, unsigned int size_, bool file_ ) :
        name( name_ ),
    disc( disc_ ),
    size( size_ ),
    file( file_ ) { };
    inline QString  getName()   { return name; }
    inline QString  getDisc()   { return disc; }
    inline unsigned int getSize()   { return size; }
    inline QString  getSizeString() {
        if( size > 1073741824 )
            return QString().setNum( size/1073741824 ).append("GB");
        if( size > 1048576 )
            return QString().setNum( size/1048576 ).append("MB");
        else if( size > 1024 )
            return QString().setNum( size/1024 ).append("KB");
        else 
            return QString().setNum( size ).append("B");
        
    }
    inline bool     isFile()    { return file; }
    private:
        QString name;
        QString disc;
        unsigned int size;
        bool file;
};

/**
@author Florian Zubeil
 */

class SearchResultList : public QPtrList<SearchResultItem>
{
    public:
        enum Fields{ None, Name, Disc, Size };
        SearchResultList() : by(None), last(None) {};

        ~SearchResultList() {};
        
        void sortBy( Fields f ) { by = f; sort(); if( last == f ) last = None; else last = f; };
        void lastSort( Fields f ) { last = f; };
        int compareItems ( QPtrCollection::Item item1, QPtrCollection::Item item2 );
        
    private:
        Fields by;
        Fields last;
        

};

class SearchResults
{
public:
    SearchResults() : numRes(0) {};
    ~SearchResults() {};
    
    
    
    inline bool addResult( SearchResultItem* res ) { 
        if(!res) return false; 
        resList.append(res); 
        numRes++; 
        return true; 
    };
    
    inline bool fillTable( QTable *table ) { 
        table->setNumRows( numRes );
        int i=0;
        for( SearchResultItem* item = resList.first(); item; item = resList.next() )
        {
            table->setText( i, 0, item->getName() );
            table->setText( i, 1, item->getDisc() );
	    table->setItem( i, 2, new SizeTableItem( table, item->getSize(), QTableItem::Never ) );
            i++;
        }
        return true;
    }

    inline void sortBy( SearchResultList::Fields f ) {resList.sortBy( f );};

private:
    SearchResultList resList;
    int numRes;
};

#endif //__SEARCHRESULTS_H__
