/***************************************************************************
 *   Copyright (C) 2005 by Florian Zubeil   *
 *   daswrack@oleco.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "StatusDialog.h"

StatusDialog::StatusDialog(QSettings& settings_, QListView& dirView_, QListView& fileView_, QWidget* parent_, const char* name, bool modal, WFlags fl ) :
              StatusDialogBase(parent_, name, modal, fl ),
              settings( settings_ ),
              dirView(dirView_),
              fileView(fileView_),
              parent(*parent_),
              abort(false),
              vReadThread(0),
              fileCount(0),
              success( false )
{
    installEventFilter(this);
}

StatusDialog::~StatusDialog()
{

}

void StatusDialog::setEnabled( bool enabled )
{
    scanButton->setEnabled( enabled );
}

void StatusDialog::scan()
{
    if( (mPointEdit->text() == "") || (labelEdit->text() == "") ) {
        QMessageBox::critical(this, "Missing Value", "Either the mountpoint or the diskname is missing!\n",
                              QMessageBox::Ok, QMessageBox::NoButton);
        return;
    }

    

    FileReadThread* readThread = new FileReadThread(settings, mPointEdit->text(), labelEdit->text(), &dirView, &fileView, &parent, this, &abort);
    vReadThread = readThread;
    vReadThread->start();
    success = true;
}

void StatusDialog::mPointBrowse()
{
    QString mPoint = QFileDialog::getExistingDirectory();
    if( mPoint != QString::null )
        mPointEdit->setText( mPoint );
}

void StatusDialog::cancel()
{
    abort = true;
    if(vReadThread)
        vReadThread->wait();
    this->close();
}

bool StatusDialog::eventFilter( QObject *o, QEvent *e )
{
    //ACE_DEBUG((LM_ERROR,ACE_TEXT("hallo\n")));
    if( o == this )
        if( e->type() == QEvent::User ) {
            progressBar->setProgress( progressBar->progress()+1 );
            return true;
        }
    return false;
}

