/***************************************************************************
 *   Copyright (C) 2005 by Florian Zubeil   *
 *   daswrack@oleco.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "XmlIo.h"

XmlIo::XmlIo(QListView* view_, QString filename_, QString pass_ ) :
            view(view_),
            filename( filename_ ),
            pass( pass_ )
{

}

XmlIo::~XmlIo()
{

}

//int XmlIo::save( QListView *view, QString filename)
int XmlIo::save( QString as, QString pass_ )
{
    QString sFile;
    QString sPass;
    if( as.isNull() )
        sFile = filename;
    else
        sFile = as;
    if( pass_.isNull() )
        sPass = pass;
    else 
        sPass = pass_;
    QListViewItemIterator it( view );
    QDomDocument doc("dvdindex");
    QDomElement root = doc.createElement("Disks");
    QListViewItem* viewItem = view->firstChild();
    while ( viewItem ) {
        if( viewItem->rtti() == RTTI_DIRVIEWITEM )
        {
            DirViewItem *item = (DirViewItem*)viewItem;
            DirItem* dirItem = (DirItem*)item->getDir();
            ACE_DEBUG((LM_DEBUG,ACE_TEXT("%N:%l: Root element: '%C'\n"),dirItem->getName().latin1()));
            QDomElement disk = doc.createElement("Disk");
            QDomAttr attr = doc.createAttribute("Name");
            attr.setValue( dirItem->getName() );
            disk.setAttributeNode( attr );
            dirItem->addToDom( &disk, &doc );

        
            root.appendChild( disk );
        }
        viewItem = viewItem->nextSibling();
    }
    doc.appendChild( root );

    int enc_len = 0;
    char * enc_buf;
    char in[doc.toString().length()];
    char cPass[sPass.length()];
    memcpy(in, doc.toString().latin1(), doc.toString().length());
    memcpy(cPass, sPass.latin1(), sPass.length() );
    enc_buf = aes_enc( in, doc.toString().length(), cPass, sPass.length(), &enc_len);

    ofstream outfile( sFile.latin1() );
    if( !outfile )
        ACE_ERROR_RETURN((LM_ERROR,ACE_TEXT("%N:%l: Error: Cannot open file '%C' for writing!\n"),filename.latin1()),-1);
    outfile.write( enc_buf, enc_len );
    outfile.close();


    return 1;
}

//int XmlIo::load( QListView *view, QString filename)
int XmlIo::load()
{
    //Load file to buffer
    ifstream infile( filename, ios::binary );
    if( !infile )
        ACE_ERROR_RETURN((LM_ERROR,ACE_TEXT("%N:%l: Error: Cannot open file '%C' for reading!\n"),filename.latin1()),-1);
    int file_length = 0;
    int dec_len;
    char cPass[pass.length()];
    memcpy(cPass, pass.latin1(), pass.length() );
    // determine file_length
    infile.seekg( 0, ios::end );
    file_length = infile.tellg();
    infile.seekg( 0, ios::beg );
    // allocate memory and read whole file to buffer buf
    char buf[ file_length ];
    infile.read( buf, file_length );
    infile.close();

    char * dec_buf;
    dec_buf = aes_dec( buf, file_length, cPass, pass.length(), &dec_len );


    // Interpret decyrpted stream
    QDomDocument doc("dvdindex");

    if( !doc.setContent( QString( dec_buf ) ) ) {
        ACE_ERROR_RETURN((LM_ERROR,ACE_TEXT("%N:%l: Error: Cannot set Content of file '%C'!\n"),filename.latin1()),-2);
    }

    QDomElement root = doc.documentElement();
    if( root.tagName() != "Disks" )
        ACE_ERROR_RETURN((LM_ERROR,ACE_TEXT("%N:%l: Error: root element name is '%C', not 'Disks'!\n"),root.tagName().latin1()),-3);

    QDomNode n = root.firstChild();

    for( int i=0; !n.isNull(); i++)
    {
        DirItem *item = new DirItem( &n, n.toElement().attribute("Name"), n.toElement().attribute("Name"), true );
        item->addListViewItem( view );
        n = n.nextSibling();
    }


    return 1;
}
